package ch.mydoli.focal.sql.database

import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns

object FeedContract {

    object FeedEntry : BaseColumns {
        const val TABLE_FEED = "feed"
        const val COLUMN_DATE = "date"
        const val COLUMN_TITLE = "title"
        const val COLUMN_DESCRIPTION = "description"
        const val COLUMN_IMAGE = "image"
        const val COLUMN_FAVORITE = "favorite"
    }

    private const val SQL_CREATE_ENTRIES =
        "CREATE TABLE ${FeedEntry.TABLE_FEED} (" +
                "${FeedEntry.COLUMN_DATE} DATE PRIMARY KEY," +      // YYYY-MM-DD format
                "${FeedEntry.COLUMN_TITLE} TEXT," +                 // up to 65,535 characters
                "${FeedEntry.COLUMN_DESCRIPTION} MEDIUMTEXT," +     // up to 16,777,215 characters
                "${FeedEntry.COLUMN_IMAGE} TINYTEXT," +             // up to 255 characters
                "${FeedEntry.COLUMN_FAVORITE} BOOLEAN NOT NULL)"    // default is false for not null boolean

    private const val SQL_DELETE_ENTRIES = "DROP TABLE IF EXISTS ${FeedEntry.TABLE_FEED}"

    private const val SQL_BACKUP_ENTRIES = "CREATE TABLE BACKUP_${FeedEntry.TABLE_FEED} AS SELECT * FROM ${FeedEntry.TABLE_FEED}"

    class FeedDbHelper(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

        companion object {
            const val DATABASE_VERSION = 1 // increment if schema changes
            const val DATABASE_NAME = "feed.sqlite"
        }

        override fun onCreate(db: SQLiteDatabase) {
            db.execSQL(SQL_CREATE_ENTRIES)
        }

        override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
            db.execSQL(SQL_BACKUP_ENTRIES)
            db.execSQL(SQL_DELETE_ENTRIES)
            onCreate(db)
            // If migration should occur at some point, write this part
        }

        override fun onDowngrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
            onUpgrade(db, oldVersion, newVersion)
            // If migration should occur at some point, write this part
        }
    }

}
