package ch.mydoli.focal.ui.photos

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.GridView
import android.widget.ImageView
import androidx.core.os.bundleOf
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import ch.mydoli.focal.R
import ch.mydoli.focal.databinding.FragmentFavoritesBinding
import com.google.android.material.bottomnavigation.BottomNavigationView
import java.io.File
import java.time.LocalDate

class FavoritesFragment : Fragment() {

    private var _binding: FragmentFavoritesBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    @SuppressLint("ClickableViewAccessibility")
    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        val photosViewModel: PhotosViewModel by viewModels()

        _binding = FragmentFavoritesBinding.inflate(inflater, container, false)
        val root: View = binding.root

        // Show bottom navigation bar
        val navView: BottomNavigationView = requireActivity().findViewById(R.id.nav_view)
        navView.visibility = View.VISIBLE

        val gridFavorites: GridView = binding.gridFavorites
        val favorites = photosViewModel.getFavorites()
        val gridAdapter = GridViewAdapter(requireContext(), favorites)
        gridFavorites.adapter = gridAdapter

        // Click on image to open day fragment
        gridFavorites.setOnItemClickListener { _, view, _, _ ->
            val date = (view as SquareImageView).date
            val bundle = bundleOf(
                DayFragment.YEAR to date.year,
                DayFragment.MONTH to date.monthValue,
                DayFragment.DAY to date.dayOfMonth
            )
            view.alpha = 0.5F
            findNavController().navigate(R.id.action_navigation_favorites_to_navigation_day, bundle)
        }

        if (favorites.isEmpty()) {
            binding.gridFeedEmpty.visibility = View.VISIBLE
        }

        navView.setOnItemReselectedListener { it ->
            gridFavorites.smoothScrollToPosition(0)
        }

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    internal class GridViewAdapter(private val localContext: Context, private val datesImages: List<Pair<LocalDate, String>>): ArrayAdapter<Any?>(localContext, 0, datesImages) {

        override fun getCount(): Int {
            return datesImages.size
        }

        override fun getItem(position: Int): Any? {
            return datesImages[position]
        }

        override fun getItemId(position: Int): Long {
            return datesImages[position].first.toEpochDay()
        }

        @SuppressLint("ClickableViewAccessibility")
        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val imageView: SquareImageView
            if (convertView == null) {
                imageView = SquareImageView(localContext)
                imageView.scaleType = ImageView.ScaleType.CENTER_CROP
                imageView.setPadding(0, 0, 0, 0)
            } else {
                imageView = convertView as SquareImageView
            }

            val (date, imageName) = datesImages[position]

            val imgFile = File(context.filesDir.toString() + "/" + date.toString() + "_" + imageName + "_thumb")
            if (imgFile.exists()) {
                val imgBitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
                imageView.setImageBitmap(imgBitmap)
            }

            imageView.alpha = 1F
            imageView.date = date

            /*imageView.setOnTouchListener { view, event ->
                var rect = Rect(view.left.toFloat() + 10f, view.top.toFloat() + 100f, view.right.toFloat() - 10f, view.bottom.toFloat() - 100f)
                when (event.action) {
                    android.view.MotionEvent.ACTION_DOWN -> {
                        view.alpha = 0.5F
                    }
                    android.view.MotionEvent.ACTION_MOVE -> {
                        if (!rect.contains(Offset(view.left + event.x, view.top + event.y))) {
                            view.alpha = 1F
                        }
                    }
                    android.view.MotionEvent.ACTION_UP -> {
                        view.alpha = 1F
                    }
                    else -> { }
                }
                false
            }*/

            return imageView
        }
    }
}