package ch.mydoli.focal.ui.photos

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.BitmapFactory
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.preference.PreferenceManager
import java.io.File
import java.time.LocalDate
import ch.mydoli.focal.R
import ch.mydoli.focal.ui.settings.SettingsFragment

class FeedEntryView @SuppressLint("ClickableViewAccessibility") constructor(
    context: Context,
    val date: LocalDate,
    title: String,
    description: String,
    imageName: String
) : LinearLayout(context) {

    private var imageView: ImageView
    private var linearLayout: LinearLayout
    private var titleView: TextView
    private var descriptionView: TextView
    private var dateView: TextView

    init {
        val view = inflate(context, R.layout.view_feed_entry, this)
        imageView = findViewById(R.id.image_view)
        linearLayout = findViewById(R.id.linear_view)
        titleView = findViewById(R.id.title_view)
        descriptionView = findViewById(R.id.description_view)
        dateView = findViewById(R.id.date_view)
        imageView.scaleType = ImageView.ScaleType.CENTER_CROP
        imageView.setPadding(0, 0, 0, 0)
        imageView.alpha = 1F
        val imgFile = File(context.filesDir.toString() + "/" + date.toString() + "_" + imageName + "_thumb")
        if (imgFile.exists()) {
            val imgBitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
            imageView.setImageBitmap(imgBitmap)
        }
        titleView.text = title
        descriptionView.text = description

        // Format date
        val sharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)
        dateView.text = SettingsFragment.getDateFormat(context, date, sharedPreferences.getString(context.getString(R.string.key_date_format), context.getString(R.string.yyyymmdd)))
    }
}