package ch.mydoli.focal.ui.photos

import android.annotation.SuppressLint
import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.GridView
import androidx.core.os.bundleOf
import androidx.core.view.MenuProvider
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.Lifecycle
import androidx.navigation.fragment.findNavController
import ch.mydoli.focal.R
import ch.mydoli.focal.databinding.FragmentFeedBinding
import com.google.android.material.bottomnavigation.BottomNavigationView
import java.io.File
import java.time.LocalDate

class FeedFragment : Fragment() {

    private var _binding: FragmentFeedBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private lateinit var newEntryMenuItem: MenuItem

    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        val photosViewModel: PhotosViewModel by viewModels()

        _binding = FragmentFeedBinding.inflate(inflater, container, false)
        val root: View = binding.root

        // Show bottom navigation bar
        val navView: BottomNavigationView = requireActivity().findViewById(R.id.nav_view)
        navView.visibility = View.VISIBLE

        val listFeed: GridView = binding.gridFeedEntries
        val feed = photosViewModel.getFeed()
        val listAdapter = ListViewAdapter(requireContext(), feed)
        listFeed.adapter = listAdapter

        val now = LocalDate.now()

        // Create toolbar menu
        requireActivity().addMenuProvider(object : MenuProvider {
            override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
                // Add menu items here
                menuInflater.inflate(R.menu.top_new_entry_menu, menu)

                newEntryMenuItem = menu.findItem(R.id.new_entry_pick)

                if (photosViewModel.entryExists(now)) {
                    val imageName = photosViewModel.getImage(now)
                    val thumbFile =
                        File(requireActivity().filesDir.toString() + "/" + now.toString() + "_" + imageName + "_thumb")
                    if (thumbFile.exists()) {
                        // If the entry is already in the list below, no need to add a new entry
                        newEntryMenuItem.isVisible = false
                    }
                }
            }

            override fun onMenuItemSelected(menuItem: MenuItem): Boolean {
                // Handle the menu selection
                return when (menuItem.itemId) {
                    R.id.new_entry_pick -> {
                        // Button to add an entry for today
                        val bundle = bundleOf(
                            DayFragment.YEAR to now.year,
                            DayFragment.MONTH to now.monthValue,
                            DayFragment.DAY to now.dayOfMonth
                        )
                        findNavController().navigate(R.id.action_navigation_feed_to_navigation_day, bundle)
                        true
                    }
                    else -> {
                        false
                    }
                }
            }
        }, viewLifecycleOwner, Lifecycle.State.RESUMED)

        // Click on image to open day fragment
        listFeed.setOnItemClickListener { _, view, _, _ ->
            val date = (view as FeedEntryView).date
            val bundle = bundleOf(
                DayFragment.YEAR to date.year,
                DayFragment.MONTH to date.monthValue,
                DayFragment.DAY to date.dayOfMonth
            )
            findNavController().navigate(R.id.action_navigation_feed_to_navigation_day, bundle)
        }

        if (feed.isEmpty()) {
            binding.gridFeedEmpty.visibility = View.VISIBLE
        }

        navView.setOnItemReselectedListener {
            listFeed.smoothScrollToPosition(0)
        }

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    internal class ListViewAdapter(private val localContext: Context, private val datesTitlesDescriptionsImages: List<Pair<Pair<LocalDate, String>, Pair<String, String>>>): ArrayAdapter<Any?>(localContext, 0, datesTitlesDescriptionsImages) {

        override fun getCount(): Int {
            return datesTitlesDescriptionsImages.size
        }

        override fun getItem(position: Int): Any {
            return datesTitlesDescriptionsImages[position]
        }

        override fun getItemId(position: Int): Long {
            return datesTitlesDescriptionsImages[position].first.first.toEpochDay()
        }

        @SuppressLint("ClickableViewAccessibility")
        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val (date, title) = datesTitlesDescriptionsImages[position].first
            val (description, imageName) = datesTitlesDescriptionsImages[position].second
            return FeedEntryView(context, date, title, description, imageName)
        }
    }
}