package ch.mydoli.focal.ui.photos

import android.annotation.SuppressLint
import android.app.Activity
import android.graphics.BitmapFactory
import android.os.Bundle
import android.transition.TransitionInflater
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import androidx.fragment.app.Fragment
import ch.mydoli.focal.R
import ch.mydoli.focal.databinding.FragmentPhotoBinding
import com.davemorrissey.labs.subscaleview.ImageSource
import com.davemorrissey.labs.subscaleview.SubsamplingScaleImageView
import com.google.android.material.bottomnavigation.BottomNavigationView
import java.io.File

class PhotoFragment : Fragment() {

    companion object {
        internal const val IMAGE = "image"
        internal const val DATE = "date"
    }

    private var _binding: FragmentPhotoBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private var isActionBarHidden = false
    private lateinit var imageView: SubsamplingScaleImageView

    @SuppressLint("ClickableViewAccessibility")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentPhotoBinding.inflate(inflater, container, false)
        val root: View = binding.root

        // Hide bottom navigation bar
        val navView: BottomNavigationView = requireActivity().findViewById(R.id.nav_view)
        navView.visibility = View.GONE

        val date = arguments?.getString(DATE)
        val imageName = arguments?.getString(IMAGE)

        imageView = binding.imageView

        val imgFile =
            File(requireActivity().filesDir.toString() + "/" + date.toString() + "_" + imageName)
        if (imgFile.exists()) {
            val imgBitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
            imageView.setImage(ImageSource.bitmap(imgBitmap))
        } else {
            imageView.setImage(ImageSource.resource(R.drawable.ic_empty_image))
        }

        // Enable zoom
        imageView.apply {
            setMinimumDpi(80)
            setDoubleTapZoomDpi(240)
            resetScaleAndCenter()
        }

        imageView.setOnClickListener {
            val windowInsetsController = WindowCompat.getInsetsController((it.context as Activity).window, it)
            // Configure the behavior of the hidden system bars
            if (isActionBarHidden) {
                windowInsetsController.systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
                // Show both the status bar and the navigation bar
                (it.context as AppCompatActivity).supportActionBar?.show()
                windowInsetsController.show(WindowInsetsCompat.Type.systemBars())
                // Set hidden toggle to false
                isActionBarHidden = false
            } else {
                // Configure the behavior of the hidden system bars
                windowInsetsController.systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
                // Hide both the status bar and the navigation bar
                (it.context as AppCompatActivity).supportActionBar?.hide()
                windowInsetsController.hide(WindowInsetsCompat.Type.systemBars())
                // Set hidden toggle to true
                isActionBarHidden = true
            }
        }

        return root
    }

    // TODO: Make transition work
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        sharedElementEnterTransition = TransitionInflater.from(requireContext())
            .inflateTransition(R.transition.shared_image)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        val windowInsetsController = WindowCompat.getInsetsController((imageView.context as Activity).window, imageView)
        windowInsetsController.systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
        // Show both the status bar and the navigation bar
        (imageView.context as AppCompatActivity).supportActionBar?.show()
        windowInsetsController.show(WindowInsetsCompat.Type.systemBars())
        _binding = null
    }
}